#!/usr/bin/env python3
# coding=utf-8
__version__ = "0.8.0"
__desc__ = "Convert markdown to html with TOC(table of contents) and Mermaid diagram support https://github.com/ltto/mdtree"
from mdtree.main import MdTree, convert_from_file
